﻿using System;
using System.Net;
using System.Windows;
using System.IO.IsolatedStorage;
using System.IO;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// Manages the caching of the downloaded and converted themes
    /// </summary>
    internal static class ThemeCacheManager
    {
        /// <summary>
        /// Checks if there is a cached and converted version of the defined thmx file
        /// </summary>
        /// <param name="uri">The Uri to the thmx file</param>
        /// <returns>A boolean indicating if there is a cached version</returns>
        public static bool IsCached(Uri uri)
        {
            bool ret = false;
            using (IsolatedStorageFile userStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                ret = userStore.FileExists(GetCacheKey(uri));
            }
            return ret;
        }

        /// <summary>
        /// Gets a Stream to a cached and converted version of the thmx file defined by the Uri
        /// </summary>
        /// <param name="uri">The Uri to the thmx file</param>
        /// <param name="contentLength">The length of the returned Stream</param>
        /// <returns>A Stream to the cached version</returns>
        public static Stream GetCachedTheme(Uri uri, out long contentLength)
        {
            if (!IsCached(uri))
                throw new ArgumentException(string.Format("Could not find a cached theme for the uri <{0}>", uri) ,"uri");

            Stream stream = null;
            using (IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication())
            using (IsolatedStorageFileStream isfs = new IsolatedStorageFileStream(GetCacheKey(uri), FileMode.Open, isf))
            {
                contentLength = isfs.Length;
                stream = new MemoryStream(new byte[contentLength], true);
                isfs.CopyTo(stream);
                stream.Seek(0, SeekOrigin.Begin);
            }
            return stream;
        }

        /// <summary>
        /// Puts a converted theme in the cache
        /// </summary>
        /// <param name="uri">The Uri to the corresponding thmx file</param>
        /// <param name="themeXaml">The string containing the converted version of the theme</param>
        public static bool CacheTheme(Uri uri, string themeXaml)
        {
            using (IsolatedStorageFile userStore = IsolatedStorageFile.GetUserStoreForApplication())
            {
                int streamLength = System.Text.Encoding.UTF8.GetByteCount(themeXaml);
                if (userStore.UsedSize + streamLength > userStore.Quota)
                {
                    OnIsolatedStorageIncreaseRequired(userStore.Quota + userStore.Quota * 5);
                    return false;
                }
                using (IsolatedStorageFileStream stream = userStore.CreateFile(GetCacheKey(uri)))
                {
                    StreamWriter sw = new StreamWriter(stream);
                    sw.Write(themeXaml);
                    sw.Close();
                }
            }
            return true;
        }

        /// <summary>
        /// Gets a cache key based on the Uri passed in
        /// </summary>
        /// <param name="uri">The Uri to the original theme file</param>
        /// <returns>A string based cache key</returns>
        private static string GetCacheKey(Uri uri)
        {
            string file = uri.AbsolutePath.TrimStart('/');
            return file.Replace(Path.GetExtension(file), ".xaml").Replace('/', '_');
        }

        /// <summary>
        /// Raises the IsolatedStorageIncreaseRequired event if any handlers have been attached
        /// </summary>
        /// <param name="requiredSize">The required cache size</param>
        private static void OnIsolatedStorageIncreaseRequired(long requiredSize)
        {
            if (IsolatedStorageIncreaseRequired != null)
                IsolatedStorageIncreaseRequired(null, new IsolatedStorageIncreaseEventArgs(requiredSize));
        }
        public static event EventHandler<IsolatedStorageIncreaseEventArgs> IsolatedStorageIncreaseRequired;
    }

    public class IsolatedStorageIncreaseEventArgs : EventArgs
    {
        public IsolatedStorageIncreaseEventArgs(long requiredSize)
        {
            RequiredSize = requiredSize;
        }

        public long RequiredSize
        {
            get;
            private set;
        }
    }
}
