﻿using System;
using System.Net;
using System.Windows;
using System.IO;
using System.Xml.Linq;
using System.Linq;
using System.Windows.Resources;
using System.Xml;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// Converts themes in thmx format to Xaml based themes
    /// </summary>
    internal class ThemeConverter
    {
        #region Constants

        public static string ThmxNamespace = @"http://schemas.openxmlformats.org/drawingml/2006/main";
        public static string ThmxThemeUri = @"theme/theme/theme1.xml";

        private static string thmx_dk1 = "dk1";
        private static string thmx_dk2 = "dk2";
        private static string thmx_lt1 = "lt1";
        private static string thmx_lt2 = "lt2";
        private static string thmx_a1 = "accent1";
        private static string thmx_a2 = "accent2";
        private static string thmx_a3 = "accent3";
        private static string thmx_a4 = "accent4";
        private static string thmx_a5 = "accent5";
        private static string thmx_a6 = "accent6";
        private static string thmx_hlink = "hlink";
        private static string thmx_flink = "folHlink";
        private static string thmx_majfont = "majorFont";
        private static string thmx_minfont = "minorFont";

        private static string xaml_dk1 = "{dk1}";
        private static string xaml_dk2 = "{dk2}";
        private static string xaml_lt1 = "{lt1}";
        private static string xaml_lt2 = "{lt2}";
        private static string xaml_a1 = "{a1}";
        private static string xaml_a2 = "{a2}";
        private static string xaml_a3 = "{a3}";
        private static string xaml_a4 = "{a4}";
        private static string xaml_a5 = "{a5}";
        private static string xaml_a6 = "{a6}";
        private static string xaml_hlink = "{hlink}";
        private static string xaml_flink = "{flink}";
        private static string xaml_majfont = "{majfont}";
        private static string xaml_minfont = "{minfont}";

        #endregion
        string _template;

        public ThemeConverter()
        {
            Stream templateStream = this.GetType().Assembly.GetManifestResourceStream("MSUS.Silverlight.Thmx2Theme.Controls.theme_template.xml");
            StreamReader sr = new StreamReader(templateStream);
            _template = sr.ReadToEnd();
            templateStream.Close();
        }

        /// <summary>
        /// Converts a thmx file to a Xaml based theme in string format
        /// </summary>
        /// <param name="thmxFile">A Stream to a thmx file</param>
        /// <returns>A string containing the Xaml for the converted theme</returns>
        public string ConvertTheme(Stream thmxFile)
        {
            StreamResourceInfo thmxSRI = new StreamResourceInfo(thmxFile, null);
            StreamResourceInfo sri = Application.GetResourceStream(thmxSRI, new Uri(ThmxThemeUri, UriKind.Relative));

            //Add the namespace manager
            NameTable nameTable = new NameTable();
            XmlNamespaceManager manager = new XmlNamespaceManager(nameTable);
            manager.AddNamespace("a", ThmxNamespace);

            //Create XDocument from thmx stream
            XDocument themeDoc = XDocument.Load(sri.Stream);
            XDocument xDocTheme = XDocument.Parse(themeDoc.ToString());

            //Get RGB Values for the 12 theme colors
            string dk1 = GetRGBFromTheme(xDocTheme, thmx_dk1);
            string dk2 = GetRGBFromTheme(xDocTheme, thmx_dk2);
            string lt1 = GetRGBFromTheme(xDocTheme, thmx_lt1);
            string lt2 = GetRGBFromTheme(xDocTheme, thmx_lt2);
            string a1 = GetRGBFromTheme(xDocTheme, thmx_a1);
            string a2 = GetRGBFromTheme(xDocTheme, thmx_a2);
            string a3 = GetRGBFromTheme(xDocTheme, thmx_a3);
            string a4 = GetRGBFromTheme(xDocTheme, thmx_a4);
            string a5 = GetRGBFromTheme(xDocTheme, thmx_a5);
            string a6 = GetRGBFromTheme(xDocTheme, thmx_a6);
            string hlink = GetRGBFromTheme(xDocTheme, thmx_hlink);
            string flink = GetRGBFromTheme(xDocTheme, thmx_flink);

            //Get the two fonts
            string majFont = GetFontFamilyFromTheme(xDocTheme, thmx_majfont);
            string minFont = GetFontFamilyFromTheme(xDocTheme, thmx_minfont);

            //Generate new xaml theme from template
            XDocument newTheme = XDocument.Parse(_template);

            //Replace xaml theme colors from thmx colors
            string rawXml = newTheme.ToString();
            rawXml = rawXml.Replace(xaml_dk1, dk1);//Text
            rawXml = rawXml.Replace(xaml_dk2, dk2);//Main Dark Color
            rawXml = rawXml.Replace(xaml_lt1, lt1);//Window Background
            rawXml = rawXml.Replace(xaml_lt2, lt2);//Main Light Color
            rawXml = rawXml.Replace(xaml_a1, a1);//Accent1
            rawXml = rawXml.Replace(xaml_a2, a2);//Accent2
            rawXml = rawXml.Replace(xaml_a3, a3);//Accent3
            rawXml = rawXml.Replace(xaml_a4, a4);//Accent4
            rawXml = rawXml.Replace(xaml_a5, a5);//Accent5
            rawXml = rawXml.Replace(xaml_a6, a6);//Accent6
            rawXml = rawXml.Replace(xaml_hlink, hlink);//Hyperlink
            rawXml = rawXml.Replace(xaml_flink, flink);//Followed Link
            rawXml = rawXml.Replace(xaml_majfont, hlink);//Heading Font
            rawXml = rawXml.Replace(xaml_minfont, flink);//Body Font

            return rawXml;
        }

        #region Helpers

        /// <summary>
        /// Gets an RGB string value from an XDocument based on an element name
        /// </summary>
        /// <param name="XDocTheme">An XDocument containing theme xml</param>
        /// <param name="ElementName">The name of the element to read the color from</param>
        /// <returns>A string containing an RGB color. 6 hexadecimal digits.</returns>
        static string GetRGBFromTheme(XDocument XDocTheme, string ElementName)
        {
            XNamespace ns = ThmxNamespace;
            try
            {
                return (from c in XDocTheme.Descendants(ns + ElementName).Descendants(ns + "srgbClr")
                        select new { c.Attribute("val").Value }).FirstOrDefault().Value;
            }
            catch
            {
                return (from c in XDocTheme.Descendants(ns + ElementName).Descendants(ns + "sysClr")
                        select new { c.Attribute("lastClr").Value }).FirstOrDefault().Value;
            }

        }

        /// <summary>
        /// Gets a font family string value from an XDocument based on an element name
        /// </summary>
        /// <param name="XDocTheme">An XDocument containing theme xml</param>
        /// <param name="ElementName">The name of the element to read the font family from</param>
        /// <returns>A string containing the name of a font family.</returns>
        static string GetFontFamilyFromTheme(XDocument XDocTheme, string ElementName)
        {
            XNamespace ns = ThmxNamespace;
            return (from c in XDocTheme.Descendants(ns + ElementName).Descendants(ns + "latin")
                    select new { c.Attribute("typeface").Value }).FirstOrDefault().Value;

        }

        #endregion
    }
}
