﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls.Theming;
using System.IO;
using System.Windows.Controls.Primitives;
using System.IO.IsolatedStorage;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// A custom Theme implementation that uses a .thmx file for the theming
    /// </summary>
    [TemplateVisualState(GroupName = "LoadingStates", Name = "Default")]
    [TemplateVisualState(GroupName = "LoadingStates", Name = "Loading")]
    [TemplateVisualState(GroupName = "LoadingStates", Name = "CacheIncreaseRequired")]
    [TemplatePart(Type = typeof(ButtonBase), Name = "CacheIncreaseButton")]
    [TemplatePart(Type = typeof(ButtonBase), Name = "CacheCleanButton")]
    public class ThmxTheme : Theme
    {
        bool _needsCacheIncrease;
        bool _isLoadingTheme;
        long _requiredCacheSize;

        public ThmxTheme()
        {
            this.DefaultStyleKey = typeof(ThmxTheme);

            ThmxLoader.Initialize(Dispatcher);
            ThmxLoader.ThemeLoaded += OnThemeLoaded;
            ThmxLoader.ThemeLoadingFailed += OnThemeLoadingFailed;

            ThemeCacheManager.IsolatedStorageIncreaseRequired += OnCacheIncreaseRequired;
        }

        /// <summary>
        /// Called when the templated is applied. Hooks up handlers for the cache modification buttons
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            ButtonBase btn = GetTemplateChild("CacheIncreaseButton") as ButtonBase;
            if (btn != null)
            {
                btn.Click += (s, e) => {
                    using (IsolatedStorageFile userStore = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        if (userStore.IncreaseQuotaTo(_requiredCacheSize))
                        {
                            _needsCacheIncrease = false;
                            base.OnThemeUriPropertyChanged(ThemeUri, ThemeUri);
                        }
                        else
                        {
                            // Failed for some reason. Ignore and show application and hope next time works better.
                            _needsCacheIncrease = false;
                            _isLoadingTheme = false;
                            SetState();
                        }
                    }
                };
            }
            btn = GetTemplateChild("CacheCleanButton") as ButtonBase;
            if (btn != null)
            {
                btn.Click += (s, e) =>
                {
                    if (MessageBox.Show("Are you sure you want to clear the cache?", "Are you sure?", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                    {
                        using (IsolatedStorageFile userStore = IsolatedStorageFile.GetUserStoreForApplication())
                        {
                            _needsCacheIncrease = false;
                            userStore.Remove();
                            base.OnThemeUriPropertyChanged(ThemeUri, ThemeUri);
                        }
                    }
                };
            }
            SetState();
        }

        /// <summary>
        /// Eventhandler for the ThmxLoaders ThemeLoaded event. 
        /// Makes sure that the control's visual state is set to NotLoading when a theme is loaded
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The parameters passed from the raising object</param>
        void OnThemeLoaded(object sender, PayloadEventArgs<Uri> e)
        {
            if (e.Payload == ThemeUri)
            {
                _isLoadingTheme = false;
                SetState();
            }
        }

        /// <summary>
        /// Eventhandler for the ThmxLoaders ThemeLoadingFailed event. 
        /// Makes sure that the control's visual state is set to NotLoading when a theme download fails.
        /// This makes sure that the "Loading..." screen is removed if the theme loading failes.
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The parameters passed from the raising object</param>
        void OnThemeLoadingFailed(object sender, PayloadEventArgs<Uri> e)
        {
            if (e.Payload == ThemeUri)
            {
                _isLoadingTheme = false;
                SetState();
            }
        }

        /// <summary>
        /// Handled the CacheIncreaseRequired event on the ThmxCacheManager, making sure that the correct VisualState is set
        /// </summary>
        /// <param name="sender">The object that raised the event. In this case it will always be null</param>
        /// <param name="e">The eventargs passed from the raising object</param>
        void OnCacheIncreaseRequired(object sender, IsolatedStorageIncreaseEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
            {
                _requiredCacheSize = e.RequiredSize;
                _needsCacheIncrease = true;
                SetState();
            });
        }

        /// <summary>
        /// Called every time the ThemeUri of the base class is changed
        /// </summary>
        /// <param name="oldValue">The value before the change</param>
        /// <param name="newValue">The value after the change</param>
        protected override void OnThemeUriPropertyChanged(Uri oldValue, Uri newValue)
        {
            _needsCacheIncrease = false;
            if (Path.GetExtension(newValue.OriginalString).Equals(".thmx", StringComparison.OrdinalIgnoreCase))
            {
                _isLoadingTheme = true;
                SetState();
                // Looking for THMX file
                if (!newValue.IsAbsoluteUri)
                {
                    Uri absoluteUri;
                    // Convert to absolut Uri with correct scheme
                    if (newValue.OriginalString.StartsWith("/"))
                        absoluteUri = new Uri(string.Format("{0}://{1}:{2}{3}", ThmxLoader.THMX_SCHEME, Application.Current.Host.Source.Host, Application.Current.Host.Source.Port, newValue.OriginalString));
                    else
                    {
                        string localPath = "";
                        int index = Application.Current.Host.Source.AbsolutePath.LastIndexOf('/');
                        if (index >= 0)
                            localPath = Application.Current.Host.Source.AbsolutePath.Substring(0, Application.Current.Host.Source.AbsolutePath.LastIndexOf('/')) + "/";
                        absoluteUri = new Uri(string.Format("{0}://{1}:{2}{3}{4}", ThmxLoader.THMX_SCHEME, Application.Current.Host.Source.Host, Application.Current.Host.Source.Port, localPath, newValue.OriginalString));
                    }
                    if (ForceThemeReload)
                    {
                        absoluteUri = new Uri(absoluteUri.ToString() + "?forceThemeReload=1");
                    }
                    this.ThemeUri = absoluteUri;
                    return;
                }
                else if (newValue.Scheme != ThmxLoader.THMX_SCHEME)
                {
                    // Rewrite the URL to the correct scheme
                    Uri absoluteUri = new Uri(newValue.ToString().ToLower().Replace("http", ThmxLoader.THMX_SCHEME));
                    if (ForceThemeReload)
                    {
                        absoluteUri = new Uri(absoluteUri.ToString() + "?forceThemeReload=1");
                    }
                    this.ThemeUri = absoluteUri;
                    return;
                }
                else
                {
                    base.OnThemeUriPropertyChanged(oldValue, newValue);
                }
            }
            else
            {
                base.OnThemeUriPropertyChanged(oldValue, newValue);
            }
        }

        /// <summary>
        /// Handles the setting of the controls visual states
        /// </summary>
        /// <param name="isLoading"></param>
        protected virtual void SetState()
        {
            if (_needsCacheIncrease)
                VisualStateManager.GoToState(this, "CacheIncreaseRequired", false);
            else if (_isLoadingTheme)
                VisualStateManager.GoToState(this, "Loading", false);
            else
                VisualStateManager.GoToState(this, "Default", false);
        }

        /// <summary>
        /// Gets or sets whether or not the defined theme should be reloaded even if available in the cache
        /// </summary>
        public bool ForceThemeReload
        {
            get;
            set;
        }
    }
}
