﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Collections.Generic;

namespace MSUS.Silverlight.Thmx2Theme.Controls
{
    /// <summary>
    /// A WebRequest implementation that handles requests to resources using the "thmx://" scheme
    /// </summary>
    internal class ThmxRequest : WebRequest
    {
        readonly Uri _requestUri;
        Dictionary<IAsyncResult, ThmxResponse> _responses = new Dictionary<IAsyncResult, ThmxResponse>();

        public ThmxRequest(Uri uri)
        {
            _requestUri = uri;
        }

        #region WebRequest overrides
        public override IAsyncResult BeginGetResponse(AsyncCallback callback, object state)
        {
            if (ThemeCacheManager.IsCached(_requestUri) && !_requestUri.Query.Contains("forceThemeReload=1"))
            {
                ThmxAsyncResult asyncResult = new ThmxAsyncResult(callback) { AsyncState = state, CompletedSynchronously = true };
                _responses.Add(asyncResult, new ThmxResponse(_requestUri));
                asyncResult.IsCompleted = true;
                return asyncResult;
            }
            else
            {
                ThmxAsyncResult asyncResult = new ThmxAsyncResult(callback) { AsyncState = state, IsCompleted = false, CompletedSynchronously = false };
                DownloadFile(asyncResult);
                return asyncResult;
            }
        }
        public override WebResponse EndGetResponse(IAsyncResult asyncResult)
        {
            if (!_responses.ContainsKey(asyncResult))
                return null;

            OnResponseRead();

            return _responses[asyncResult];
        }

        #region Not Implemented

        public override void Abort()
        {
            throw new NotImplementedException();
        }

        public override IAsyncResult BeginGetRequestStream(AsyncCallback callback, object state)
        {
            throw new NotImplementedException();
        }

        public override Stream EndGetRequestStream(IAsyncResult asyncResult)
        {
            throw new NotImplementedException();
        }

        public override WebHeaderCollection Headers
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override string Method
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override string ContentType
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        #endregion

        public override Uri RequestUri
        {
            get
            {
                return _requestUri;
            }
        }

        #endregion

        /// <summary>
        /// Downloads a .thmx file, converts it to Xaml using a ThemeConverter and then puts it in the cache using the ThemeCacheManager 
        /// </summary>
        /// <param name="asyncResult">The ThmxAsyncResult corresponding to the download request</param>
        private void DownloadFile(ThmxAsyncResult asyncResult)
        {
            string file = _requestUri.AbsoluteUri.Replace("thmx://", "http://").TrimEnd('/');

            WebRequest request = WebRequest.Create(new Uri(file));
            request.BeginGetResponse((iar) => {
                WebResponse response = null;
                try
                {
                    response = request.EndGetResponse(iar);
                }
                catch 
                {
                    OnDownloadFailed();
                    return; 
                }

                ThemeConverter converter = new ThemeConverter();

                Stream responseStream = response.GetResponseStream();
                string xamlTheme = converter.ConvertTheme(responseStream);
                responseStream.Close();

                if (ThemeCacheManager.CacheTheme(request.RequestUri, xamlTheme))
                {
                    _responses.Add(asyncResult, new ThmxResponse(request.RequestUri));
                }
                else
                {
                    MemoryStream stream = new MemoryStream(System.Text.Encoding.UTF8.GetBytes(xamlTheme));
                    _responses.Add(asyncResult, new ThmxResponse(stream));
                }
                asyncResult.IsCompleted = true;
            }, null);
        }

        /// <summary>
        /// Raises the ResponseRead event if any handlers have been attached
        /// </summary>
        protected virtual void OnResponseRead()
        {
            if (ResponseRead != null)
                ResponseRead(this, new PayloadEventArgs<Uri>(_requestUri));
        }

        /// <summary>
        /// Raised when the response from the request has been read
        /// </summary>
        public event EventHandler<PayloadEventArgs<Uri>> ResponseRead;

        /// <summary>
        /// Raises the DownloadFailed event if any handlers have been attached
        /// </summary>
        protected virtual void OnDownloadFailed()
        {
            if (DownloadFailed != null)
                DownloadFailed(this, new PayloadEventArgs<Uri>(_requestUri));
        }

        /// <summary>
        /// Raised if download of the file has failed
        /// </summary>
        public event EventHandler<PayloadEventArgs<Uri>> DownloadFailed;
    }
}
